@@ -96,10 +96,12 @@ _gcry_kem_genkey (int algo,
           || pubkey_len != GCRY_KEM_SNTRUP761_PUBKEY_LEN)
         return GPG_ERR_INV_ARG;
       sntrup761_keypair (pubkey, seckey, NULL, sntrup761_random);
+      _gcry_burn_stack (SNTRUP761_KEYPAIR_STACK_BURN);
       return 0;
 
     case GCRY_KEM_CM6688128F:
       mceliece6688128f_keypair (pubkey, seckey);
+      _gcry_burn_stack (MCELIECE6688128F_KEYPAIR_STACK_BURN);
       return 0;
 
 #if USE_KYBER
@@ -109,6 +111,7 @@ _gcry_kem_genkey (int algo,
           || (optional && optional_len != GCRY_KEM_MLKEM_RANDOM_LEN*2))
         return GPG_ERR_INV_ARG;
       kyber_keypair (algo, pubkey, seckey, optional);
+      _gcry_burn_stack (KYBER_KEYPAIR_STACK_BURN (algo));
       return 0;
 
     case GCRY_KEM_MLKEM768:
@@ -117,6 +120,7 @@ _gcry_kem_genkey (int algo,
           || (optional && optional_len != GCRY_KEM_MLKEM_RANDOM_LEN*2))
         return GPG_ERR_INV_ARG;
       kyber_keypair (algo, pubkey, seckey, optional);
+      _gcry_burn_stack (KYBER_KEYPAIR_STACK_BURN (algo));
       return 0;
 
     case GCRY_KEM_MLKEM1024:
@@ -125,6 +129,7 @@ _gcry_kem_genkey (int algo,
           || (optional && optional_len != GCRY_KEM_MLKEM_RANDOM_LEN*2))
         return GPG_ERR_INV_ARG;
       kyber_keypair (algo, pubkey, seckey, optional);
+      _gcry_burn_stack (KYBER_KEYPAIR_STACK_BURN (algo));
       return 0;
 #endif
 
@@ -166,12 +171,14 @@ _gcry_kem_encap (int algo,
           || shared_len != GCRY_KEM_SNTRUP761_SHARED_LEN)
         return GPG_ERR_INV_VALUE;
       sntrup761_enc (ciphertext, shared, pubkey, NULL, sntrup761_random);
+      _gcry_burn_stack (SNTRUP761_ENC_STACK_BURN);
       return 0;
 
     case GCRY_KEM_CM6688128F:
       if (optional != NULL)
 	return GPG_ERR_INV_VALUE;
       mceliece6688128f_enc (ciphertext, shared, pubkey);
+      _gcry_burn_stack (MCELIECE6688128F_ENC_STACK_BURN);
       return 0;
 
 #if USE_KYBER
@@ -181,6 +188,7 @@ _gcry_kem_encap (int algo,
       if (optional && optional_len != GCRY_KEM_MLKEM_RANDOM_LEN)
 	return GPG_ERR_INV_VALUE;
       kyber_encap (algo, ciphertext, shared, pubkey, optional);
+      _gcry_burn_stack (KYBER_ENCAP_STACK_BURN (algo));
       return 0;
 #endif
 
@@ -228,12 +236,14 @@ _gcry_kem_decap (int algo,
           || shared_len != GCRY_KEM_SNTRUP761_SHARED_LEN)
         return GPG_ERR_INV_VALUE;
       sntrup761_dec (shared, ciphertext, seckey);
+      _gcry_burn_stack (SNTRUP761_DEC_STACK_BURN);
       return 0;
 
     case GCRY_KEM_CM6688128F:
       if (optional != NULL)
 	return GPG_ERR_INV_VALUE;
       mceliece6688128f_dec (shared, ciphertext, seckey);
+      _gcry_burn_stack (MCELIECE6688128F_DEC_STACK_BURN);
       return 0;
 
 #if USE_KYBER
@@ -243,6 +253,7 @@ _gcry_kem_decap (int algo,
       if (optional != NULL)
         return GPG_ERR_INV_VALUE;
       kyber_decap (algo, shared, ciphertext, seckey);
+      _gcry_burn_stack (KYBER_DECAP_STACK_BURN (algo));
       return 0;
 #endif
 
