@@ -319,7 +319,7 @@ static struct RI {
     {	"/usr/bin/lpstat", "-t", SC(0.1), NULL, 0, 0, 0, 1 },
     {	"/usr/ucb/lpstat", "-t", SC(0.1), NULL, 0, 0, 0, 0 },
     {	"/usr/bin/tcpdump", "-c 5 -efvvx", SC(1), NULL, 0, 0, 0, 0 },
-    /* This is very environment-dependant.  If network traffic is low, it'll
+    /* This is very environment-dependent.  If network traffic is low, it'll
      * probably time out before delivering 5 packets, which is OK because
      * it'll probably be fixed stuff like ARP anyway */
     {	"/usr/sbin/advfsstat", "-b usr_domain",
@@ -714,7 +714,7 @@ start_gatherer( int pipefd )
     int dbgall;
 
     {
-	const char *s = getenv("GNUPG_RNDUNIX_DBG");
+	const char *s = getenv("GCRYPT_RNDUNIX_DBG");
 	if( s ) {
 	    dbgfp = (*s=='-' && !s[1])? stdout : fopen(s, "a");
 	    if( !dbgfp )
@@ -723,7 +723,7 @@ start_gatherer( int pipefd )
 	    else
 		fprintf(dbgfp,"\nSTART RNDUNIX DEBUG pid=%d\n", (int)getpid());
 	}
-	dbgall = !!getenv("GNUPG_RNDUNIX_DBGALL");
+	dbgall = !!getenv("GCRYPT_RNDUNIX_DBGALL");
     }
     /* close all files but the ones we need */
     {	int nmax, n1, n2, i;
@@ -894,7 +894,7 @@ _gcry_rndunix_gather_random (void (*add)(const void*, size_t,
     /* now read from the gatherer */
     while( length ) {
 	int goodness;
-	ulong subtract;
+	unsigned long subtract;
 
 	if( read_a_msg( pipedes[0], &msg ) ) {
 	    log_error("reading from gatherer pipe failed: %s\n",
@@ -928,7 +928,7 @@ _gcry_rndunix_gather_random (void (*add)(const void*, size_t,
 	(*add)( msg.data, n, origin );
 
 	/* this is the trick how we cope with the goodness */
-	subtract = (ulong)n * goodness / 100;
+	subtract = (unsigned long)n * goodness / 100;
 	/* subtract at least 1 byte to avoid infinite loops */
 	length -= subtract ? subtract : 1;
     }
